/**
	Description of this class...
	@author Toni Thomsson, toni@tonjac.org
*/

#include <zz_string.h>

#include <shoutcc.h>
#include <stdlib.h>
#include <fcntl.h>
#include <io.h>	
#include "mp3_source_config.h"

int main( int argc, char** argv )
{
	Shout s;
	ShoutMetadata md;
	static unsigned char buff[4096]; 
	long read = 0;
	long total = 0;
	int size = 0;
	bool http = true;

	try
	{
        CzzString m_User = getenv( "REMOTE_USER" );
        CzzString mount = CzzString::Format( "%s/%s", Cmp3SourceConfig::getSourceMount().c_str(), m_User.c_str() );

		if( Cmp3SourceConfig::getServerProtocol() == "http" )
			http = true;
		else
			http = false;

		s.set_user( Cmp3SourceConfig::getSourceUser().c_str() );
		s.set_password( Cmp3SourceConfig::getSourcePwd().c_str() );
		
		if( http )
			s.set_mount( mount.c_str() );

		s.set_port( atoi( Cmp3SourceConfig::getServerPort().c_str() ) );
		s.set_host( Cmp3SourceConfig::getServerHost().c_str() );
		s.set_format( SHOUT_FORMAT_MP3 );

		if( http )
			s.set_protocol( SHOUT_PROTOCOL_HTTP );
		else
			s.set_protocol( SHOUT_PROTOCOL_ICY );

		s.set_name( CzzString::Format( "%s - %s", Cmp3SourceConfig::getSourceName().c_str(), m_User.c_str() ).c_str() );
		s.set_audio_info( SHOUT_AI_BITRATE, Cmp3SourceConfig::getSourceBitrate().c_str() );
		
		s.open(); 

		int fn = fileno( stdin );
		int mode = setmode( fn, _O_BINARY );

		while( (read = fread( buff, 1, sizeof(buff), stdin )) > 0 ) 
		{
			s.send(buff, read);
			s.sync();
		}
	}
	catch( ShoutErr& serr )
	{
		printf( "Couldn't stream %s, exiting stream thread", serr.err.c_str() );	
	}
	catch( ... )
	{
		printf( "Couldn't stream (undefined reason), exiting stream thread");
	}

	s.close();

	return 0;
}