/* myMusic
 *
 * This program is distributed under the GNU General Public License, version 2.
 * A copy of this license is included with this source.
 *
 * Copyright 2004-2006, Toni Thomsson <toni@tonjac.org> 
*/

// Implementation of class Cmp3HtmlServiceManager

#include "mp3_http_requester.h"
#include "mp3_html_service_manager.h"
#include <zz_file.h>
#include <zz_path.h>
#include <zz_win_exception.h>
#include <zz_string_tokenizer.h>
#include <time.h>
#include <sys/timeb.h>
#include <sys/types.h> 
#include <id3/tag.h>
#include <zz_file.h>
#include <winver.h>


#include "mp3_artist_list_render.h"
#include "mp3_record_list_render.h"
#include "mp3_song_list_render.h"
#include "mp3_myradio_render.h"
#include "mp3_radio_list_render.h"
#include "mp3_playlist_render.h"
#include "mp3_latest_list_render.h"
#include "mp3_playlist_content_render.h"
#include "mp3_seek_result_render.h"
#include "mp3_info_render.h"
#include "mp3_help_render.h"
#include "mp3_frames_render.h"
#include "mp3_general_render.h"

#ifdef _WIN32
	#include <process.h>
	#include <direct.h>
#else
	#include <stdlib.h>
#endif

// Construction
Cmp3HtmlServiceManager::Cmp3HtmlServiceManager( const CzzString& session )
 : m_Log( "mp3_gui" ), m_CGI(), m_Session( session ), m_User( ), m_IP()
{
	
}

// Destruction
Cmp3HtmlServiceManager::~Cmp3HtmlServiceManager()
{
	
}

// Execute service !
void Cmp3HtmlServiceManager::execute( void )
{

	checkIP();
	
	//checkLicense();
	parseCmd();
}

// Find service to call
void Cmp3HtmlServiceManager::parseCmd( void )
{
	int svc = atoi( m_CGI[ "svc" ].c_str() );
	
	CzzOutputRender* r = NULL;

	// Find service to call
	switch( svc )
	{
		// Render frames
		case MP3_SVC_FRAMES:
			r = new Cmp3FramesRender( &m_Session, &m_CGI );
			printf( "%s", r->render().c_str() );
			break;

		// Render top frame
		case MP3_SVC_TOP:
			r = new Cmp3GeneralRender( &m_Session, &m_CGI, "renderTop.html" );
			printf( "%s", r->render().c_str() );
			break;

		// Render left frame
		case MP3_SVC_LEFT:
			r = new Cmp3GeneralRender( &m_Session, &m_CGI, "renderLeft.html" );
			printf( "%s", r->render().c_str() );
			break;

		// List all artists
		case MP3_SVC_LIST_ARTISTS:
			r = new Cmp3ArtistListRender( &m_Session, &m_CGI );
			printf( "%s", r->render().c_str() );
			break;
		
		// List all records for a artist
		case MP3_SVC_LIST_RECORDS:
			r = new Cmp3RecordListRender( &m_Session, &m_CGI );
			printf( "%s", r->render().c_str() );		
			break;

		// List all songs on a record
		case MP3_SVC_LIST_SONGS:
			r = new Cmp3SongListRender( &m_Session, &m_CGI );
			printf( "%s", r->render().c_str() );
			break;

		// Add song to radio queue
		case MP3_SVC_LIST_RADIO_ADD:
			r = new Cmp3SongListRender( &m_Session, &m_CGI, true );
			printf( "%s", r->render().c_str() );
			break;

		// Add song to playlist
		case MP3_SVC_LIST_M3U_ADD:
			r = new Cmp3SongListRender( &m_Session, &m_CGI, false, true );
			printf( "%s", r->render().c_str() );
			break;

		// Add record to the radio queue
		case MP3_SVC_LIST_RADIO_ADD_RECORD:
			r = new Cmp3SongListRender( &m_Session, &m_CGI, false, false, true );
			printf( "%s", r->render().c_str() );
			break;

		// List radio queue
		case MP3_SVC_LIST_MYRADIO:
			r = new Cmp3MyRadioRender( &m_Session, &m_CGI );
			printf( "%s", r->render().c_str() );
			break;

		// Clear radio queue
		case MP3_SVC_LIST_RADIO_CLEAR:
			checkAdmin();
			r = new Cmp3MyRadioRender( &m_Session, &m_CGI, MP3_SVC_LIST_RADIO_CLEAR );
			printf( "%s", r->render().c_str() );
			break;

		// Randomize the radio queue
		case MP3_SVC_LIST_RADIO_RAND:
			checkAdmin();
			r = new Cmp3MyRadioRender( &m_Session, &m_CGI, MP3_SVC_LIST_RADIO_RAND );
			printf( "%s", r->render().c_str() );
			break;

		// Skip one song forward in the radio queue
		case MP3_SVC_LIST_RADIO_FORWARD:
			checkAdmin();
			r = new Cmp3MyRadioRender( &m_Session, &m_CGI, MP3_SVC_LIST_RADIO_FORWARD );
			printf( "%s", r->render().c_str() );
			break;

		// Skip one song backward in the radio queue
		case MP3_SVC_LIST_RADIO_REVERSE:
			checkAdmin();
			r = new Cmp3MyRadioRender( &m_Session, &m_CGI, MP3_SVC_LIST_RADIO_REVERSE );
			printf( "%s", r->render().c_str() );
			break;

		// Remove radio queue
		case MP3_SVC_LIST_RADIO_REMOVE:
			checkAdmin();
			r = new Cmp3MyRadioRender( &m_Session, &m_CGI, MP3_SVC_LIST_RADIO_REMOVE );
			printf( "%s", r->render().c_str() );
			break;

		// List Radio channels
		case MP3_SVC_LIST_RADIO:
			r = new Cmp3RadioListRender( &m_Session, &m_CGI );
			printf( "%s", r->render().c_str() );
			break;

		// List all m3u lists
		case MP3_SVC_LIST_M3U:
			r = new Cmp3PlaylistRender( &m_Session, &m_CGI );
			printf( "%s", r->render().c_str() );
			break;

		// List latest updates
		case MP3_SVC_LIST_LATEST:
			r = new Cmp3LatestListRender( &m_Session, &m_CGI );
			printf( "%s", r->render().c_str() );
			break;

		// List content of a m3u list
		case MP3_SVC_LIST_M3U_CONTENTS:
			r = new Cmp3PlaylistContentRender( &m_Session, &m_CGI, MP3_SVC_LIST_M3U_CONTENTS );
			printf( "%s", r->render().c_str() );
			break;

		// Empty playlist
		case MP3_SVC_LIST_M3U_DELETE:
			r = new Cmp3PlaylistContentRender( &m_Session, &m_CGI, MP3_SVC_LIST_M3U_DELETE );
			printf( "%s", r->render().c_str() );
			break;

		// Save playlist
		case MP3_SVC_LIST_M3U_SAVE:
			r = new Cmp3PlaylistContentRender( &m_Session, &m_CGI, MP3_SVC_LIST_M3U_SAVE );
			printf( "%s", r->render().c_str() );
			break;

		// Remove song from playlist
		case MP3_SVC_LIST_M3U_REMOVE:
			r = new Cmp3PlaylistContentRender( &m_Session, &m_CGI, MP3_SVC_LIST_M3U_REMOVE );
			printf( "%s", r->render().c_str() );
			break;

		// Replace radio queue
		case MP3_SVC_LIST_RADIO_REPLACE:
			checkAdmin();
			r = new Cmp3PlaylistContentRender( &m_Session, &m_CGI, MP3_SVC_LIST_RADIO_REPLACE );
			printf( "%s", r->render().c_str() );
			break;

		// Seek
		case MP3_SVC_LIST_SEEK:
			r = new Cmp3SeekResultRender( &m_Session, &m_CGI );
			printf( "%s", r->render().c_str() );
			break;

		// Info
		case MP3_SVC_LIST_INFO:
			r = new Cmp3InfoRender( &m_Session, &m_CGI );
			printf( "%s", r->render().c_str() );
			break;
		
		// Help
		case MP3_SVC_HELP:
			r = new Cmp3HelpRender( &m_Session, &m_CGI );
			printf( "%s", r->render().c_str() );
			break;
		
		// Not a valid service call!
		default:
			throw CzzLogException( _LAF, 4711, "Unknown service" );
	}

	if( r )
		delete r;
}

CzzString Cmp3HtmlServiceManager::getUser( void )
{

	if( m_User.size() <= 0 )
	{
		char* u = getenv( "REMOTE_USER" );
	
		if( u != NULL )
			m_User = u;
		else
			m_User = "unknown";
	}

	return m_User;
}

CzzString Cmp3HtmlServiceManager::getIP( void )
{

	if( m_IP.size() <= 0 )
	{
		char* u = getenv( "REMOTE_ADDR" );
	
		if( u != NULL )
			m_IP = u;
		else
			m_IP = "unknown";
	}

	return m_IP;
}

void Cmp3HtmlServiceManager::checkAdmin( void )
{
	/*
	if( Cmp3Config::getAdminUser( getUser() ) == 0 )
		throw CzzLogException( _LAF, 4711, "Can only be done by admin" );*/
}

void Cmp3HtmlServiceManager::checkIP( void )
{
	CzzString ip = Cmp3Config::getIPUser( getUser() );

	if( ip.size() > 0 )
	{
		if( ip.find( getIP() ) == -1 )
			throw CzzLogException( _LAF, 4711, CzzString::Format( "User access restricted to IP: %s", ip.c_str() ) );
	}

}

void Cmp3HtmlServiceManager::checkLicense( void )
{
	Cmp3HttpRequester req( "mymusic.tonjac.org", "80" );
	CzzString lnr = Cmp3Config::getLicenseNo();

	CzzString reply = req.checkLicense( lnr );

	if( reply.find( "OK" ) == -1 )
		throw CzzLogException( _LAF, 4711, "Licenserror" );
}

