/* myMusic
 *
 * This program is distributed under the GNU General Public License, version 2.
 * A copy of this license is included with this source.
 *
 * Copyright 2004-2006, Toni Thomsson <toni@tonjac.org> 
*/

// Implementation of class Cmp3PlaylistContentListData

#include "mp3_playlist_content_list_data.h"
#include "mp3_svc.h"
#include "mp3_config.h"

#include <time.h>
#include <sys/timeb.h>
#include <sys/types.h> 
#include <id3/tag.h>
#include <zz_file.h>
#include <winver.h>

// Construction
Cmp3PlaylistContentListData::Cmp3PlaylistContentListData( const CzzString& name  )
: CzzDataService( "listPlaylistContentData" ), m_Name(name)
{

}

// Destruction
Cmp3PlaylistContentListData::~Cmp3PlaylistContentListData()
{

}

CzzDataServiceReply* Cmp3PlaylistContentListData::execute( void* s, CzzParameterMap* req )
{
	CzzDataService::execute( s, req );

	CzzDataServiceReply* reply = new CzzDataServiceReply();

	reply->addFieldName( MP3_FLD_REMOVE_SONG );
	reply->addFieldName( MP3_FLD_SONG_TITLE );
	reply->addFieldName( MP3_FLD_YEAR );
	reply->addFieldName( MP3_FLD_BITRATE );
	reply->addFieldName( MP3_FLD_SONG_LENGTH );

	int size = 0;
	CzzString path = Cmp3Config::getM3uDir();
	CzzString cgi = Cmp3Config::getCGIDir();
	CzzString root = Cmp3Config::getRootDir();
	char buff[2048];
	FILE* f = NULL;
	bool edit = false;
	CzzString name = m_Name;
    CzzString session = *((CzzString*)s);

	if( name == "session" )
	{
		f = fopen( (".\\" + session + ".m3u" ).c_str(), "r" );
		edit = true;
	}
	else
		f = fopen( (path + "\\" + name ).c_str(), "r" );

	if( f != NULL )
	{
		while( fgets( buff, 2048, f ) != NULL )
		{
			if( buff[0] != '#' )
			{
				buff[strlen(buff)-1] = '\0';
				size += printM3uSong( name, buff, reply );
			}
		}
			
		fclose( f );
	}
	
	return reply;
}

int Cmp3PlaylistContentListData::printM3uSong( const CzzString& name, const char* line, CzzDataServiceReply* reply )
{
	int result = 0;
	CzzString cgi = Cmp3Config::getCGIDir();
	CzzString path = Cmp3Config::getDataDir();
	char* song = new char[ strlen(line)+1 ];
	
	for( int i = strlen(line)-1; i >= 0; i-- )
	{
		if( line[i] == '\\' )
		{
			strcpy( song, line+i+1 );
			break;
		}
	}

	char* s = new char[strlen(song)+1];
	strcpy(s,song);
	s[ strlen( song )-4 ] = '\0';
	uint32 minutes = 0;
	uint32 secs = 0;
	uint32 speed = 128;
	CzzString year("-");

	try
	{
		ID3_Frame* frame = NULL;
		ID3_Field* fld = NULL;
		ID3_Tag tag( line );
		result = tag.GetFileSize();
		const Mp3_Headerinfo* hdr = tag.GetMp3HeaderInfo();
		if( hdr != NULL )
		{
			minutes = hdr->time/60;
			secs = hdr->time - (minutes*60);
			if( hdr->bitrate != 0 )
				speed = hdr->bitrate/1000;
			if( hdr->vbr_bitrate != 0 )
				speed = hdr->vbr_bitrate/1000;
		}
		
		frame = tag.Find( ID3FID_YEAR );
		
		if( frame != NULL )
		{
			fld = frame->GetField( ID3FN_TEXT );
    
			if( fld != NULL )
			{
				fld->SetEncoding(ID3TE_ISO8859_1);
				const char* p = fld->GetRawText();

				if( p != NULL && strlen(p) == 4 && (p[0]=='1' || p[0]=='2') )
					year = p;
			}
		}
	}
	catch( ... ) {}
	
	/*
	reply->addFieldName( MP3_FLD_REMOVE_SONG );
	reply->addFieldName( MP3_FLD_SONG_TITLE );
	reply->addFieldName( MP3_FLD_YEAR );
	reply->addFieldName( MP3_FLD_BITRATE );
	reply->addFieldName( MP3_FLD_SONG_LENGTH );
	*/

	reply->addFieldValue( CzzString::Format( "%s/mp3_gui.exe?svc=%d&name=%s&line=%s", Cmp3Config::getCGIDir().c_str(), MP3_SVC_LIST_M3U_REMOVE, name.c_str(), line ));
	reply->addFieldValue( s );
	reply->addFieldValue( year );
	reply->addFieldValue( CzzString::Format( "%03d kbit/s", speed ) );
	reply->addFieldValue( CzzString::Format( "%02d:%02d", minutes, secs ) );
	
	delete [] s;
	delete [] song;

	return result;
}

int Cmp3PlaylistContentListData::size()
{
    return sizeof(Cmp3PlaylistContentListData);
}