/* myMusic
 *
 * This program is distributed under the GNU General Public License, version 2.
 * A copy of this license is included with this source.
 *
 * Copyright 2004-2006, Toni Thomsson <toni@tonjac.org> 
*/

// Implementation of class Cmp3SeekResultData

#include "mp3_seek_result_data.h"
#include "mp3_svc.h"
#include "mp3_config.h"

#include <time.h>
#include <sys/timeb.h>
#include <sys/types.h> 
#include <id3/tag.h>
#include <zz_file.h>
#include <winver.h>

// Construction
Cmp3SeekResultData::Cmp3SeekResultData( void )
: CzzDataService( "listSeekResultData" )
{

}

// Destruction
Cmp3SeekResultData::~Cmp3SeekResultData()
{

}

CzzDataServiceReply* Cmp3SeekResultData::execute( void* session, CzzParameterMap* req )
{

	CzzDataService::execute( session, req );

	CzzDataServiceReply* reply = new CzzDataServiceReply();

	reply->addFieldName( MP3_FLD_LIST_ART );
	reply->addFieldName( MP3_FLD_TITLE_ROW );
	reply->addFieldName( MP3_FLD_INFO_ROW );
	reply->addFieldName( MP3_FLD_TITLE_LINK );
	reply->addFieldName( MP3_FLD_PLAY_LINK );


	CzzString str = (*req)[ "seek" ];
	CzzString root = Cmp3Config::getDataDir();
	CzzString cgi = Cmp3Config::getCGIDir();
	Cmp3List list(false);
	Cmp3ListItem* idx = NULL;
	CzzString album;
	CzzString artist;
	CzzString song;
	int n = 0;
	createSeekList( root.c_str(), &list );

	int c = 0;
	int max = Cmp3Config::getMaxResultSize();

	while( (idx = list.next()) != list.last() )
	{
		if( idx->getName().UpperCase().find( str.UpperCase() ) != -1 && c <= max )
		{
			artist = getArtist( idx->getName().c_str() );
			album = getAlbum( idx->getName().c_str() );
			song = getSong( idx->getName().c_str() );

			reply->addFieldValue( urlEncode( CzzString::Format( "%s/%s/%s/Folder.jpg", Cmp3Config::getMusicRoot().c_str(),
				artist.c_str(), album.c_str() ) )) ;
			
			reply->addFieldValue( CzzString::Format( "%s - %s", artist.c_str(), album.c_str() ) );

			reply->addFieldValue( CzzString::Format( "%s", song.c_str() ) );


			reply->addFieldValue( CzzString::Format( "%s/mp3_gui.exe?svc=%d&name=%s&album=%s", Cmp3Config::getCGIDir().c_str(),
				MP3_SVC_LIST_SONGS, urlEncode(artist).c_str(), urlEncode(album).c_str() ) );

			reply->addFieldValue( CzzString::Format( "%s/mp3_playlist.exe?svc=%d&name=%s&album=%s&song=%s.mp3", Cmp3Config::getCGIDir().c_str(),
				2, urlEncode(artist).c_str(), urlEncode(album).c_str(), urlEncode(song).c_str() ) );

			c++;
		}
	}

	return reply;
}

void Cmp3SeekResultData::createSeekList( const char* path, Cmp3List* list )
{
	_finddata_t finfo;
	long handle = _findfirst( (CzzString(path) + "\\*.*").c_str(), &finfo );
	int error = 0;
	
	// Read the dir
	if( (handle!= ENOENT)  && (handle != EINVAL) && (handle != -1)  )
	{
		while( !error )
		{
			// We're not interested in the . and .. 
			if( (stricmp( finfo.name, "." ) != 0) && (stricmp( finfo.name, ".." ) != 0) )
			{
				if( (finfo.attrib & _A_SUBDIR) && !(finfo.attrib & _A_SYSTEM) )
					createSeekList( (CzzString(path) + "\\" + CzzString(finfo.name)).c_str(), list );
				else
				{
					if( CzzString(finfo.name).substr( strlen(finfo.name)-4, strlen(finfo.name)-1 ) == CzzString(".mp3") )
						list->add( (CzzString(path) + "\\" + CzzString(finfo.name)).c_str(), finfo.time_write );
				}

			}
			// Next file
			error = _findnext( handle, &finfo );
		}
	}

	_findclose( handle );

}

CzzString Cmp3SeekResultData::getArtist( const char* path )
{
	int c = 0;
	int p = 0;
	CzzString result;

	
	for( int i = strlen(path)-1; i >= 0; i-- )
	{
		if( path[i] == '\\' )
		{
			c++;
			if( c == 3 )
			{
				result = CzzString(path).substr( i+1, p-i-1 );			
			}
			if( c == 2 )
				p = i;
		}
	}
	
	return result;
}

CzzString Cmp3SeekResultData::getAlbum( const char* path )
{
	int c = 0;
	int p = 0;
	CzzString result;

	
	for( int i = strlen(path)-1; i >= 0; i-- )
	{
		if( path[i] == '\\' )
		{
			c++;
			if( c == 2 )
			{
				result = CzzString(path).substr( i+1, p-i-1 );			
			}
			if( c == 1 )
				p = i;
		}
	}
	
	return result;
}

CzzString Cmp3SeekResultData::getSong( const char* path )
{
	int c = 0;
	int p = 0;
	CzzString result;

	
	for( int i = strlen(path)-1; i >= 0; i-- )
	{
		if( path[i] == '\\' )
		{
			c++;
			if( c == 1 )
			{
				result = CzzString(path).substr( i+1, strlen(path)-i-5 );			
			}
		}
	}
	
	return result;
}

int Cmp3SeekResultData::size()
{
    return sizeof(Cmp3SeekResultData);
}