/* myMusic
 *
 * This program is distributed under the GNU General Public License, version 2.
 * A copy of this license is included with this source.
 *
 * Copyright 2004-2006, Toni Thomsson <toni@tonjac.org> 
*/

// Implementation of class Cmp3SongListRender

#include "mp3_http_requester.h"
#include "mp3_song_list_render.h"
#include "mp3_general_data.h"
#include "mp3_config.h"
#include "mp3_song_list_data.h"
#include "mp3_record_data.h"
#include "mp3_svc.h"

#include <zz_file.h>
#include <zz_path.h>
#include <zz_win_exception.h>
#include <zz_string_tokenizer.h>
#include <time.h>
#include <sys/timeb.h>
#include <sys/types.h> 
#include <id3/tag.h>
#include <zz_file.h>
#include <winver.h>
  
#ifdef _WIN32
	#include <process.h>
	#include <direct.h>
#else
	#include <stdlib.h>
#endif
 
// Construction
Cmp3SongListRender::Cmp3SongListRender( void* s, CzzParameterMap* req, bool add_radio, bool add_m3u, bool add_record )
: CzzOutputRender( CzzString::Format( ".\\%s\\%s\\renderSongs.html", Cmp3Config::getLang().c_str(), Cmp3Config::getSkin().c_str() ) , s, req )
{
	CzzString cgi = Cmp3Config::getCGIDir();
	CzzString name = (*req)[ MP3_CGI_PAR_NAME ];
	CzzString album = (*req)[ MP3_CGI_PAR_ALBUM ];
	CzzString back_url = CzzString::Format( "%s/mp3_gui.exe?svc=%d&name=%s&album=%s", cgi.c_str(), MP3_SVC_LIST_RECORDS, urlEncode(name).c_str(), urlEncode(album).c_str() );
    CzzString session = *((CzzString*)s);

	this->addFieldSvc( new Cmp3GeneralData( back_url, "myMusic", 
		CzzString::Format( "%s - %s", name.c_str(), album.c_str() )) );
	
	if( add_radio ) // Add song to radio queue
	{
		CzzString song = (*req)[ "song" ];
		CzzString add;
		Cmp3HttpRequester req;
		req.addSongToRadioQueue( name.c_str(), album.c_str(), song.c_str() );
		add = song;
		this->addFieldSvc( new Cmp3RecordData( add ) );
	}
	else if( add_m3u ) // Add song to new playlis
	{
		CzzString song = (*req)[ "song" ];
		CzzString path = Cmp3Config::getDataDir();
		FILE* pl = fopen( (".\\" + session + ".m3u").c_str(), "a" );

		if( pl != NULL )
		{
			fprintf( pl, "%s\n", (path + "\\" + name + "\\" + album + "\\" + song).c_str() );
			fclose( pl );
			updateCache( session, true, name, album, song );
		}

		this->addFieldSvc( new Cmp3RecordData() );
	}
	else if( add_record ) // Add record to radio queue
	{
		CzzString song;
		CzzString add;

		Cmp3HttpRequester req;
		CzzString path = Cmp3Config::getDataDir();
		long handle = 0;
		int error = 0;
		_finddata_t finfo;
			
		// Find first file
		handle = _findfirst( (path + "\\" + name + "\\" + album + "\\*.mp3" ).c_str(), &finfo );

		// Read the dir
		if( (handle!= ENOENT)  && (handle != EINVAL) && (handle != -1) )
		{
			while( !error )
			{
				// We're not interested in the . and .. 
				if( (stricmp( finfo.name, "." ) != 0) && (stricmp( finfo.name, ".." ) != 0) )
				{
					// Is it a subdir?
					if( !(finfo.attrib & _A_SUBDIR) && !(finfo.attrib & _A_SYSTEM) )
						req.addSongToRadioQueue( name.c_str(), album.c_str(), finfo.name );
				}

				// Next file
				error = _findnext( handle, &finfo );
			}
		}

		_findclose( handle );
		add = CzzString::Format( "Album: %s", album.c_str() );
		this->addFieldSvc( new Cmp3RecordData( add ) );
	}
	else // Just render record with songlist
		this->addFieldSvc( new Cmp3RecordData() );
	
	this->addListSvc( MP3_FLD_TABLE, new Cmp3SongListData( ) );
}

// Destruction
Cmp3SongListRender::~Cmp3SongListRender()
{

}

void Cmp3SongListRender::updateCache( const CzzString& session, bool add, const CzzString& line )
{
	unsigned int sec = 0;
	unsigned int antal = 0;
	unsigned int size = 0;
	struct _stat info;
	char str[10];
	
	size = GetPrivateProfileInt("playlistinfo","size",0,(".\\" + session + ".ini").c_str());
	sec = GetPrivateProfileInt("playlistinfo","seconds",0,(".\\" + session + ".ini").c_str());
	antal = GetPrivateProfileInt("playlistinfo","number",0,(".\\" + session + ".ini").c_str());
	
	int r = _stat( line.c_str(), &info );
	
	if( r != -1 )
	{
		ID3_Tag tag( line.c_str() );
		const Mp3_Headerinfo* hdr = tag.GetMp3HeaderInfo();
		if( hdr != NULL )
		{
			if( add )
			{
				size = size + info.st_size;
				sec = sec + hdr->time;
				antal = antal + 1;
			}
			else
			{
				size = size - info.st_size;
				sec = sec - hdr->time;
				antal = antal - 1;
			}
		}

		char tmp[1024];
		char* s = NULL;
		strcpy( tmp, line.c_str() );

		for( int i = strlen(tmp)-1; i >= 0; i-- )
		{
			if( tmp[i] == '\\' )
			{
				s = tmp + i + 1;
				break;
			}
		}

		s[ strlen(s)-4 ] = '\0';

		WritePrivateProfileString( "playlistinfo","size", itoa( size, str, 10), (".\\" + session + ".ini").c_str() );
		WritePrivateProfileString( "playlistinfo","seconds", itoa( sec, str, 10), (".\\" + session + ".ini").c_str() );
		WritePrivateProfileString( "playlistinfo","number", itoa( antal, str, 10), (".\\" + session + ".ini").c_str() );
		
		if( add )
			WritePrivateProfileString( "playlistinfo","latest", s, (".\\" + session + ".ini").c_str() );
		else
			WritePrivateProfileString( "playlistinfo","latest", (CzzString("borttag av: ") + s).c_str(), (".\\" + session + ".ini").c_str() );
	}
}

void Cmp3SongListRender::updateCache( const CzzString& session, bool add, const CzzString& name, const CzzString& album, const CzzString& song )
{
	
	CzzString path = Cmp3Config::getDataDir();

	CzzString line = (path + "\\" + name + "\\" + album + "\\" + song);

	updateCache( session, add, line );
}