/* myMusic
 *
 * This program is distributed under the GNU General Public License, version 2.
 * A copy of this license is included with this source.
 *
 * Copyright 2004-2006, Toni Thomsson <toni@tonjac.org> 
*/

/**
	m3u-playlist generator
	Toni Thomsson, toni@tonjac.org
*/

#include "mp3_list.h"
#include "mp3_m3u_playlist.h"
#include <mp3_config.h>

#include <zz_file.h>
#include <zz_path.h>
#include <zz_zip.h>
#include <zz_cgi.h>
#include <zz_mime_body.h>
#include <zz_session_cookie.h>

#include <time.h>
#include <sys/timeb.h>
#include <sys/types.h> 
#include <direct.h>
#include <rpcdce.h>
#include <id3/tag.h>

void packPlayList( const CzzString& name );
void generatePlayListForArtist( const CzzString& name );
void generatePlayListForRecord( const CzzString& name, const CzzString& album );
void generatePlayListForSong( const CzzString& name, const CzzString& album, const CzzString& song );
void generateWwwPlayList( const CzzString& name );
void generateLocalRadioPlayList( const char* usr );
void generateRandomizedM3u( void );
void createFileList( const char* path, Cmp3List* list );
void getRadioM3u( const char* name );
CzzString readID3( const char* file, uint32* t );

static CzzString SESSION;

// Main
int main( int argc, char** argv )
{
    CzzSessionCookie* sc = NULL;

	try
	{
       
#ifdef BUILD_MYMUSIC_DEMO

	printf( "Content-Type: text/plain\n\n" );
	printf( "This is only a DEMO, which can't play music" );

#else
		//int r = putenv( "QUERY_STRING=svc=1&name=Marco%20Borsato&album=Luid%20en%20Duidelijk" );

		char* cookie = getenv( "HTTP_COOKIE" );
		
		if( cookie == NULL || strlen(cookie) == 0 )
        {
    		sc = new CzzSessionCookie( "TONJAC_MP3_COOKIE", "/" );
            printf( "Set-Cookie: %s\n", sc->getSetHeader().c_str() );
        }
		else
			sc = new CzzSessionCookie( cookie );

        SESSION = sc->getId();

        CzzString usr;
		CzzCGI cgi;
		long svc = atoi( cgi[ "svc" ].c_str() );
		
		if( svc < 20 )
			printf( "Content-Type: audio/x-mpegurl\n\n" );
		else
			printf( "Content-Type: application/zip\n\n" );

		switch(svc)
		{
			case 0:
				generatePlayListForArtist( cgi[ "name" ] );
				break;
			case 1:
				generatePlayListForRecord( cgi[ "name" ], cgi[ "album" ] );
				break;
			case 2:
				generatePlayListForSong( cgi[ "name" ], cgi[ "album" ], cgi["song"] );
				break;
			case 3:
				generateWwwPlayList( cgi[ "name" ] );
				break;
			case 4:
				generateRandomizedM3u( );
				break;
			case 5:
				getRadioM3u( cgi[ "name" ].c_str() );
				break;
			case 6:
                try{ usr = cgi["usr"]; } catch( ... ) { usr = getenv( "REMOTE_USER" ); }
				generateLocalRadioPlayList( usr.c_str() );
				break;
			case 20:
				packPlayList( cgi[ "name" ] );
				break;
			default:
				break;
		}
#endif

	}
	catch( CzzException& e )
	{
		e;
		return 1;
	}
	catch( ... )
	{
		return 1;
	}

	return 0;
}

void generateLocalRadioPlayList( const char* usr )
{
	Cmp3Config config;
	printf( "#EXTM3U\n" );
	printf( "#EXTINF:-1,%s - %s\n", config.getLocalRadioName().c_str(), usr );
	printf( "%s/%s", config.getLocalRadioUrl().c_str(), usr );
}

void packPlayList( const CzzString& name )
{
	Cmp3Config config;
	CzzString dir;
	CzzString n;
	
	if( name == "session" )
	{
		n = SESSION + ".m3u";
		dir = ".\\";
	}
	else
	{
		n = name;
		dir = config.getM3uDir();
	}

	char file[2048];
	FILE* f = fopen( (dir + "\\" + n).c_str(), "r" );
	CzzZip zip( (".\\" + SESSION + ".zip").c_str() );

	if( f != NULL )
	{
		while( fgets( file, 2048, f ) != NULL )
		{
			if( file[0] != '#' )
			{
				file[strlen(file)-1]='\0';
				zip.AddFile( file );
			}
		}
	}

	zip.Compress();
	fclose(f);

	struct _stat info;
	_stat( (".\\" + SESSION + ".zip").c_str(), &info );
	printf( "Content-Length: %d\n\n", info.st_size );

	f = fopen( (".\\" + SESSION + ".zip").c_str(), "rb" );
	int s = 0;
	char buff[10240];
	int fn = fileno( stdout );
	int mode = setmode( fn, _O_BINARY );

	while( (s = fread( buff, 1, 10240, f )) > 0 )
		fwrite( buff, 1, s, stdout );

	fclose(f);
	unlink( (".\\" + SESSION + ".zip").c_str() );

}

void generateWwwPlayList( const CzzString& name )
{
	Cmp3Config config;
	CzzString dir;
	CzzString n;
	CzzString song;
	int j = 0;
	
	if( name == "session" )
	{
		n = SESSION + ".m3u";
		dir = ".\\";
	}
	else
	{
		n = name;
		dir = config.getM3uDir();
	}

	CzzString host = config.getWWWHost();
	CzzString url;

	char file[2048];
	FILE* f = fopen( (dir + "\\" + n).c_str(), "r" );
	int p = 0;
	CzzString localpath;
	uint32 t;

	if( f != NULL )
	{
        printf( "#EXTM3U\n" );

		while( fgets( file, 2048, f ) != NULL )
		{
			if( file[0] != '#' )
			{
				file[strlen(file)-1]='\0';
				song = readID3( file, &t );
				
				p = 0;
				for( int i = strlen(file); i >= 0; i-- )
				{
					if( file[i] == '\\' )
					{
						p++;
						file[i] = '/';
						if( p == 3 )
						{
							j++;
							localpath = file+i+1;
							url = host + "/mp3/" + localpath;
							printf( "#EXTINF:%d,%s\n", t, song.c_str() );
                            printf( "%s\n", Cmp3M3UPlayList::urlEncode(url).c_str() );
						}
					}
				}
			}
		}

		fclose( f ) ;
	}
}

void generatePlayListForArtist( const CzzString& name )
{
	Cmp3M3UPlayList list( name );
    printf( "%s", list.toString().c_str() );
}

void generatePlayListForRecord( const CzzString& name, const CzzString& album )
{
	Cmp3M3UPlayList list( name, album );
    printf( "%s", list.toString().c_str() );
}

void generatePlayListForSong( const CzzString& name, const CzzString& album, const CzzString& song )
{
	Cmp3M3UPlayList list( name, album, song );
    printf( "%s", list.toString().c_str() );
}

void generateRandomizedM3u( void )
{
    CzzString user = getenv( "REMOTE_USER" );
	int megs = 650;
	struct tm* pt;
	time_t t;
	time( &t );                
	pt = localtime( &t );
	
	Cmp3List* list = new Cmp3List();
	Cmp3Config cfg;
	CzzString path = cfg.getDataDir();
	CzzString m3u = cfg.getM3uDir();

	CzzString playlist = CzzString::Format( "Random mix %4d-%02d-%02d %02d.%02d.%02d - %s.m3u", 
				pt->tm_year+1900, pt->tm_mon+1, pt->tm_mday, 
				pt->tm_hour, pt->tm_min, pt->tm_sec, user.c_str() );


	createFileList( path.c_str(), list );

	
	FILE* f = fopen( (m3u + "\\" + playlist).c_str(), "a" );

	if( f != NULL )
	{
		srand( (unsigned)time( NULL ) );
		int i = 0; 
		int rno = 0;
		struct _stat info;
		float k = (float)((float)RAND_MAX/(float)list->size());

		while( i < (megs*1024*1024) )
		{
			rno = (rand()/k);
			_stat( list->get( rno )->c_str(), &info );
			i = i + info.st_size;
			if( i < (megs*1024*1024) )
				fprintf( f, "%s\n", list->get( rno )->c_str() ); 
		}
		
		delete list;
		fclose(f);

		generateWwwPlayList( playlist.c_str() );
	}
}
 
void createFileList( const char* path, Cmp3List* list )
{
	_finddata_t finfo;
	long handle = _findfirst( (CzzString(path) + "\\*.*").c_str(), &finfo );
	int error = 0;

	// Read the dir
	if( (handle!= ENOENT)  && (handle != EINVAL) && (handle != -1) )
	{
		while( !error )
		{
			// We're not interested in the . and .. 
			if( (stricmp( finfo.name, "." ) != 0) && (stricmp( finfo.name, ".." ) != 0) )
			{
				if( (finfo.attrib & _A_SUBDIR) && !(finfo.attrib & _A_SYSTEM) )
					createFileList( (CzzString(path) + "\\" + CzzString(finfo.name)).c_str(), list );
				else
				{
					if( CzzString(finfo.name).substr( strlen(finfo.name)-4, strlen(finfo.name)-1 ) == CzzString(".mp3") )
						list->add( (CzzString(path) + "\\" + CzzString(finfo.name)).c_str() );
				}

			}
			// Next file
			error = _findnext( handle, &finfo );
		}
	}

	_findclose( handle );
}

void getRadioM3u( const char* name )
{
	Cmp3Config cfg;
	CzzString path = cfg.getRadioDir();	
	FILE* f = fopen( (path + "\\" + name).c_str(), "r" );
	char buff[2048];

	if( f != NULL )
	{
		while( fgets( buff, 2048, f ) != NULL )
			printf( "%s", buff );

		fclose(f);
	}
}

CzzString readID3( const char* file, uint32* t )
{
	CzzString result;
	ID3_Frame* frame = NULL;
	ID3_Field* fld = NULL;
	ID3_Tag tag( file );
	const Mp3_Headerinfo* hdr = tag.GetMp3HeaderInfo();
	CzzString title;
	CzzString artist;
	uint32 minutes = 0;
	uint32 secs = 0;
	uint32 speed = 128;

	if( hdr != NULL )
	{
		*t = hdr->time;
		minutes = hdr->time/60;
		secs = hdr->time - (minutes*60);
		if( hdr->bitrate != 0 )
			speed = hdr->bitrate/1000;
		if( hdr->vbr_bitrate != 0 )
			speed = hdr->vbr_bitrate/1000;
	}

	frame = tag.Find( ID3FID_TITLE );

	if( frame != NULL )
	{
		fld = frame->GetField( ID3FN_TEXT );

		if( fld != NULL )
		{
            fld->SetEncoding(ID3TE_ISO8859_1);
			title = fld->GetRawText();

            frame = tag.Find( ID3FID_LEADARTIST );

	        if( frame != NULL )
	        {
                fld->SetEncoding(ID3TE_ISO8859_1);
		        fld = frame->GetField( ID3FN_TEXT );

		        if( fld != NULL )
		        {
                    fld->SetEncoding(ID3TE_ISO8859_1);
			        artist = fld->GetRawText();
		        }
	        }
		}
        else
            title = CzzPath::strip( file );
	}
    else
        title = CzzPath::strip( file );
	
	if( artist.size() )
		result = CzzString::Format( "%s - %s", artist.c_str(), title.c_str() );
	else
		result = CzzString::Format( "%s", title.c_str() );;

	return result;
}