/* myMusic
 *
 * This program is distributed under the GNU General Public License, version 2.
 * A copy of this license is included with this source.
 *
 * Copyright 2004-2006, Toni Thomsson <toni@tonjac.org> 
*/

// Implementation of class Cmp3HttpServer

#include "mp3_http_server.h"
#include "mp3_web_config.h"
#include "mp3_worker_thread.h"

// Construction
Cmp3HttpServer::Cmp3HttpServer( CzzAccessLog* log )
: CzzHTTPServer( Cmp3WebConfig::getListenPort().c_str(), SOMAXCONN, log ), m_Services()
{
	for( short i = 0; i < MAXTHREADS; i++ )
		m_Services[ i ] = NULL;
}

// Destruction
Cmp3HttpServer::~Cmp3HttpServer()
{

}

void Cmp3HttpServer::OnStart()
{
	while( !m_StopRequest ) 
	{
		try{ listen(); } catch( ... ) { m_Log.Print( _INFOPARAMS, "Couldn't read request, dropped connection" ); }
	}
}

// Listen for a request
void Cmp3HttpServer::listen( void )
{
    CzzHTTPRequest* request = NULL;
    Cmp3WorkerThread* thread = NULL;

	request = waitForRequest();

	if( request )
	{
		thread = new Cmp3WorkerThread( m_AccessLog, request );
		updateThreadList( thread );
		thread->Run();
	}
}

void Cmp3HttpServer::updateThreadList( CzzWinThread* newjob )
{

	bool inserted = false;

	try
	{
		m_Log.Print( _TRACEPARAMS, "Updateing list over active threads..." );

		// Find free slot i list
		for( int i = 0; i < MAXTHREADS; i++ )
		{
			m_Log.Print( _TRACEPARAMS, "Checking slot %d", i );

			// Valid pointer
			if( m_Services[ i ] != NULL )
			{
				// Is the jobs still running?
				if( !m_Services[ i ]->Running() )
				{
					m_Log.Print( _TRACEPARAMS, "Re-using slot %d", i );
					m_Log.Print( _TRACEPARAMS, "Removing thread %d at [%d]", m_Services[ i ], i );

					delete m_Services[ i ];

					m_Log.Print( _TRACEPARAMS, "Adding thread %d at [%d]", newjob, i );
					m_Services[ i ] = newjob;
					inserted = true;
				}
			}
			else
			{
				m_Log.Print( _TRACEPARAMS, "New slot %d", i );

				m_Log.Print( _TRACEPARAMS, "Adding thread %d at [%d]", newjob, i );
				m_Services[ i ] = newjob;
				inserted = true;
			}

           if( inserted )
                break;
		}

		// List full!, try again!
		if( !inserted )
		{
			m_Log.Print( _WARNINGPARAMS, "Threadlist full, trying again..." );
			updateThreadList( newjob );
		}
	}
	catch( CzzException& e )
	{
		m_Log.Print( _WARNINGPARAMS, "%s", e.GetReason().c_str() );
	}
	catch( ... )
	{
		m_Log.Print( _WARNINGPARAMS, "ERROR" );
	}
}