/* libzzinet
 *
 * This program is distributed under the GNU General Public License, version 2.
 * A copy of this license is included with this source.
 *
 * Copyright 2000-2006, Toni Thomsson <toni@tonjac.org> 
*/

#ifndef zz_image_gif_h
#define zz_image_gif_h

#include <zz_export.h>

#ifdef _WIN32
extern "C"
{
#include <gd.h>
}
#else
#include <gd.h>
#endif

#include "zz_image_color.h"
#include "zz_image_font.h"

/*
	Description:
	GIF bild

	Remarks:
	Bygger p gd 1.3 av: Boutell com.inc. (opensource)

	Library:
	libgd.a libzzinet.a (HPUX) libgd.lib libzzinet.lib (win32)

	Platform:
    HPUX, win32

	Author:
	Toni Thomsson, toni@tonjac.org
*/
class ZZ_API CzzImageGif
{
public:

	/*
		Description:
		Skapar en bild-buffert i minnet

		Remarks:
		Skapar en "tom" bild av en viss storlek med en
		bakgrundsfrg

		Throws:
		CzzException
	*/
	CzzImageGif( int width,                         // IN, bredd i pixels
					int height,                     // IN, hjd i pixels
					const CzzImageColor& bg_color   // IN, bakgrundsfrg
					);
	/*
		Remarks:
		Skapar en bild mha. en lagrad bild
	*/
	CzzImageGif( const char* file // IN, gif fil
				);

	/*
		Remarks:
		Kopieringskonstruktor
	*/
	CzzImageGif( const CzzImageGif& obj );

	// Description: Frigr bild bufferten
	virtual ~CzzImageGif();

	/*
		Description:
		Skriver ut text i bilden p en viss position

		Throws:
		CzzException
	*/
	void drawText( int x,						    	// IN, x-position
					int y,						    	// IN, y-position
					const CzzImageFont& font,		    // IN, font
					const ZZ_IMAGE_FONT_ORIENTATION& o,	// IN, vertikalt/horisontellt
                    const CzzImageColor& color,         // IN, tecken frg
					const char* text				    // IN, text
					);

	/*
		Description:
		Spara bilden till en fil
		
		Throws:
		CzzException
	*/
	void dump( const char* file // IN, skvg till fil
				) const;
	
	/*
		Description:
		Klistrar in en bild p aktuell bild
		
		Remarks:
		Klistrar in bilden p aktuell bild enligt 
		angiven position och frndring av storlek

		Throws:
		CzzException
	*/
	void paste( int x,						// IN, x-position
				int y,						// IN, y-position
				int width,					// IN, bredd
				int height,					// IN, hjd
				const CzzImageGif& image	// IN, bild
				);
	
	// Description: Bredd i pixels
	int getWidth( void ) const;

	// Description: Hjd i pixels
	int getHeight( void ) const;

	// Description: Bakgrundsfrg
	const CzzImageColor& getBackgroundColor( void ) const;
	
	/*
		Description:
		Tilldelningsoperator
	*/
	const CzzImageGif& operator=( const CzzImageGif& obj // IN, objekt som skall kopieras
                                );

protected:

	/*
		Description:
		Ta reda p frgindex som bst verenstmmer
		med angiven frg

		Remarks:
		protected
	*/
    int findColor( const CzzImageColor& color );

	/*
		Description:
		Returnerar en pekare till angiven font
				
		Remarks:
		protected
	*/
    gdFontPtr findFontPtr( const CzzImageFont& font );
    
private:

	gdImagePtr m_Image;
};

#endif // zz_image_gif_h



